// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file PetscSolver.h */

#ifndef AMDIS_PETSC_SOLVER_H
#define AMDIS_PETSC_SOLVER_H

#include "parallel/PetscProblemStat.h"
#include "CreatorInterface.h"

namespace AMDiS {

  using namespace std;

  class PetscSolver
  {
  protected:
    PetscSolver(Mat& petscMatrix, MeshDistributor *dist, int n) 
      : matrix(petscMatrix),
	meshDistributor(dist),
	nComponents(n)
    {}

  public:
    virtual ~PetscSolver() {}

    /// Is used to create a PETSc solver and preconditioner object. This function may
    /// be overridden to create some special PETSc solver.
    virtual void providePetscSolver(KSP &solver, PC &pc) = 0;

    static PetscSolver* getPetscSolver(Mat& petscMatrix, 
				       MeshDistributor *dist, 
				       int n);

  protected:
    Mat& matrix;

    MeshDistributor *meshDistributor;

    int nComponents;
  };


  class PetscSolverGlobalMatrix : public PetscSolver
  {
  public:
    PetscSolverGlobalMatrix(Mat& petscMatrix, MeshDistributor *dist, int n)
      : PetscSolver(petscMatrix, dist, n)
    {}

    void providePetscSolver(KSP &solver, PC &pc);
  };

#ifdef HAVE_PETSC_DEV
  class PetscSolverSchur : public PetscSolver
  {
  public:
    PetscSolverSchur(Mat& petscMatrix, MeshDistributor *dist, int n)
      : PetscSolver(petscMatrix, dist, n)
    {}

    void providePetscSolver(KSP &solver, PC &pc);
  };
#endif

} // namespace AMDiS

#endif
